import java.util.Random;

import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicLong;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;


public class EnergySource {

  public final ReadWriteLock monitor = new ReentrantReadWriteLock();

  private final Random r = new Random();

  private final long MAXLEVEL = 100;

  private long level = MAXLEVEL;
  private long usage = 0;

  private ScheduledFuture<?> replenishTask; 

  private static final ScheduledExecutorService replenishTimer =
    Executors.newScheduledThreadPool(10);

  private EnergySource() {
  }

  private void init() { 
    replenishTask =
      replenishTimer.scheduleAtFixedRate(
        new Runnable() {
          public void run() {
            replenish();
          }
        }, 0, 1, TimeUnit.SECONDS);
  }

  public static EnergySource create() {
    final EnergySource src = new EnergySource();
    src.init();
    return src;
  }

  public static void shutdown() {
    replenishTimer.shutdown();
  }

  public long getUnitsAvailable() {
    monitor.readLock().lock();
    try {
      return level;
    } finally {
      monitor.readLock().unlock();
    }
  }

  public long getUsageCount() {
    monitor.readLock().lock();
    try {
      return usage;
    } finally {
      monitor.readLock().unlock();
    }
  }

  public boolean useEnergy(final long units) {
    monitor.writeLock().lock();
    try {
      if (units > 0 && level >= units) {
        level -= units;
        usage++;
        return true;
      } else {
        return false;
      }
    } finally {
      monitor.writeLock().unlock();
    }
  }

  public synchronized void stopEnergySource() {
    replenishTask.cancel(false);
  }

  private void replenish() {
    monitor.writeLock().lock();
    try {
      if (level < MAXLEVEL) {
        long delta = (long)r.nextInt((int)(MAXLEVEL - level)) + 1;
        long newAmount = level + delta;
        newAmount = Math.min(100, newAmount);
        level = newAmount;
      }
    } finally {
      monitor.writeLock().unlock();
    }
  }

}
